@@ -2,7 +2,7 @@
  * Simulator of microcontrollers (hc08cl.h)
  *
  * Copyright (C) 1999,99 Drotos Daniel, Talker Bt.
- * 
+ *
  * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
  *
  */
@@ -44,7 +44,7 @@ public:
   class cl_memory *rom;
   struct t_regs regs;
 public:
-  cl_hc08(class cl_sim *asim);
+  cl_hc08(int Itype, int Itech, class cl_sim *asim);
   virtual int init(void);
   virtual const char *id_string(void);
 
@@ -56,7 +56,7 @@ public:
   virtual int inst_length(t_addr addr);
   virtual int inst_branch(t_addr addr);
   virtual int longest_inst(void);
-  virtual const char *disass(t_addr addr, char *sep);
+  virtual const char *disass(t_addr addr, const char *sep);
   virtual void print_regs(class cl_console_base *con);
 
   virtual int exec_inst(void);
