@@ -73,6 +73,8 @@ cnvToFcall (iCode * ic, eBBlock * ebp)
       bitVectUnSetBit (OP_USES (left), ic->key);
   if (IS_SYMOP (right))
       bitVectUnSetBit (OP_USES (right), ic->key);
+  if (IS_SYMOP (IC_RESULT (ic)))
+      bitVectUnSetBit (OP_DEFS (IC_RESULT (ic)), ic->key);
 
   if (IS_FLOAT (operandType (right)))
     {
@@ -228,7 +230,7 @@ cnvToFcall (iCode * ic, eBBlock * ebp)
   newic = newiCode (CALL, operandFromSymbol (func), NULL);
   IC_RESULT (newic) = IC_RESULT (ic);
   bitVectUnSetBit (OP_DEFS (IC_RESULT (ic)), ic->key);
-  OP_USES (IC_RESULT (newic)) = bitVectSetBit (OP_USES (IC_RESULT (newic)), newic->key);
+  OP_DEFS (IC_RESULT (newic)) = bitVectSetBit (OP_DEFS (IC_RESULT (newic)), newic->key);
   newic->filename = filename;
   newic->lineno = lineno;
   newic->parmBytes += bytesPushed;
@@ -270,8 +272,13 @@ cnvToFloatCast (iCode * ic, eBBlock * ebp)
   ip = ic->next;
   /* remove it from the iCode */
   remiCodeFromeBBlock (ebp, ic);
+  if (IS_SYMOP (IC_RIGHT (ic)))
+      bitVectUnSetBit (OP_USES (IC_RIGHT (ic)), ic->key);
+  if (IS_SYMOP (IC_RESULT (ic)))
+      bitVectUnSetBit (OP_DEFS (IC_RESULT (ic)), ic->key);
+
   /* depending on the type */
-  for (bwd = 0; bwd < 3; bwd++)
+  for (bwd = 0; bwd < 4; bwd++)
     {
       for (su = 0; su < 2; su++)
         {
@@ -285,7 +292,7 @@ cnvToFloatCast (iCode * ic, eBBlock * ebp)
 
   if (compareType (type, fixed16x16Type) == 1)
     {
-      func = fp16x16conv[0][3][0];
+      func = fp16x16conv[0][4][0];
       goto found;
     }
 
@@ -309,6 +316,8 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = linenno;
+      if (IS_SYMOP (IC_RIGHT (ic)))
+          OP_USES (IC_RIGHT (ic)) = bitVectSetBit (OP_USES (IC_RIGHT (ic)), newic->key);
     }
   else
     {
@@ -327,6 +336,8 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = linenno;
+      if (IS_SYMOP (IC_RIGHT (ic)))
+          OP_USES (IC_RIGHT (ic)) = bitVectSetBit (OP_USES (IC_RIGHT (ic)), newic->key);
     }
 
   /* make the call */
@@ -355,6 +366,8 @@ found:
   addiCodeToeBBlock (ebp, newic, ip);
   newic->filename = filename;
   newic->lineno = linenno;
+  if (IS_SYMOP (IC_RESULT (ic)))
+    OP_DEFS (IC_RESULT (ic)) = bitVectSetBit (OP_DEFS (IC_RESULT (ic)), newic->key);
 }
 
 /*----------------------------------------------------------------------*/
@@ -373,8 +386,13 @@ cnvToFixed16x16Cast (iCode * ic, eBBlock * ebp)
   ip = ic->next;
   /* remove it from the iCode */
   remiCodeFromeBBlock (ebp, ic);
+  if (IS_SYMOP (IC_RIGHT (ic)))
+      bitVectUnSetBit (OP_USES (IC_RIGHT (ic)), ic->key);
+  if (IS_SYMOP (IC_RESULT (ic)))
+      bitVectUnSetBit (OP_DEFS (IC_RESULT (ic)), ic->key);
+
   /* depending on the type */
-  for (bwd = 0; bwd < 3; bwd++)
+  for (bwd = 0; bwd < 4; bwd++)
     {
       for (su = 0; su < 2; su++)
         {
@@ -405,6 +423,8 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = linenno;
+      if (IS_SYMOP (IC_RIGHT (ic)))
+          OP_USES (IC_RIGHT (ic)) = bitVectSetBit (OP_USES (IC_RIGHT (ic)), newic->key);
     }
   else
     {
@@ -423,6 +443,8 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = linenno;
+      if (IS_SYMOP (IC_RIGHT (ic)))
+          OP_USES (IC_RIGHT (ic)) = bitVectSetBit (OP_USES (IC_RIGHT (ic)), newic->key);
     }
 
   /* make the call */
@@ -451,6 +473,8 @@ found:
   addiCodeToeBBlock (ebp, newic, ip);
   newic->filename = filename;
   newic->lineno = linenno;
+  if (IS_SYMOP (IC_RESULT (ic)))
+    OP_DEFS (IC_RESULT (ic)) = bitVectSetBit (OP_DEFS (IC_RESULT (ic)), newic->key);
 }
 
 /*-----------------------------------------------------------------*/
@@ -470,9 +494,13 @@ cnvFromFloatCast (iCode * ic, eBBlock * ebp)
   ip = ic->next;
   /* remove it from the iCode */
   remiCodeFromeBBlock (ebp, ic);
+  if (IS_SYMOP (IC_RIGHT (ic)))
+      bitVectUnSetBit (OP_USES (IC_RIGHT (ic)), ic->key);
+  if (IS_SYMOP (IC_RESULT (ic)))
+      bitVectUnSetBit (OP_DEFS (IC_RESULT (ic)), ic->key);
 
   /* depending on the type */
-  for (bwd = 0; bwd < 3; bwd++)
+  for (bwd = 0; bwd < 4; bwd++)
     {
       for (su = 0; su < 2; su++)
         {
@@ -503,6 +531,8 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = lineno;
+      if (IS_SYMOP (IC_RIGHT (ic)))
+          OP_USES (IC_RIGHT (ic)) = bitVectSetBit (OP_USES (IC_RIGHT (ic)), newic->key);
     }
   else
     {
@@ -521,6 +551,8 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = lineno;
+      if (IS_SYMOP (IC_RIGHT (ic)))
+          OP_USES (IC_RIGHT (ic)) = bitVectSetBit (OP_USES (IC_RIGHT (ic)), newic->key);
     }
 
   /* make the call */
@@ -549,6 +581,8 @@ found:
   addiCodeToeBBlock (ebp, newic, ip);
   newic->filename = filename;
   newic->lineno = lineno;
+  if (IS_SYMOP (IC_RESULT (ic)))
+    OP_DEFS (IC_RESULT (ic)) = bitVectSetBit (OP_DEFS (IC_RESULT (ic)), newic->key);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -568,9 +602,13 @@ cnvFromFixed16x16Cast (iCode * ic, eBBlock * ebp)
   ip = ic->next;
   /* remove it from the iCode */
   remiCodeFromeBBlock (ebp, ic);
+  if (IS_SYMOP (IC_RIGHT (ic)))
+      bitVectUnSetBit (OP_USES (IC_RIGHT (ic)), ic->key);
+  if (IS_SYMOP (IC_RESULT (ic)))
+      bitVectUnSetBit (OP_DEFS (IC_RESULT (ic)), ic->key);
 
   /* depending on the type */
-  for (bwd = 0; bwd < 3; bwd++)
+  for (bwd = 0; bwd < 4; bwd++)
     {
       for (su = 0; su < 2; su++)
         {
@@ -584,7 +622,7 @@ cnvFromFixed16x16Cast (iCode * ic, eBBlock * ebp)
 
   if (compareType (type, floatType) == 1)
     {
-      func = fp16x16conv[1][3][0];
+      func = fp16x16conv[1][4][0];
       goto found;
     }
 
@@ -608,6 +646,8 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = lineno;
+      if (IS_SYMOP (IC_RIGHT (ic)))
+          OP_USES (IC_RIGHT (ic)) = bitVectSetBit (OP_USES (IC_RIGHT (ic)), newic->key);
     }
   else
     {
@@ -626,6 +666,8 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = lineno;
+      if (IS_SYMOP (IC_RIGHT (ic)))
+          OP_USES (IC_RIGHT (ic)) = bitVectSetBit (OP_USES (IC_RIGHT (ic)), newic->key);
     }
 
   /* make the call */
@@ -654,6 +696,8 @@ found:
   addiCodeToeBBlock (ebp, newic, ip);
   newic->filename = filename;
   newic->lineno = lineno;
+  if (IS_SYMOP (IC_RESULT (ic)))
+    OP_DEFS (IC_RESULT (ic)) = bitVectSetBit (OP_DEFS (IC_RESULT (ic)), newic->key);
 }
 
 extern operand *geniCodeRValue (operand *, bool);
@@ -673,11 +717,21 @@ convilong (iCode * ic, eBBlock * ebp)
   int bwd;
   int su;
   int bytesPushed=0;
+  operand *left;
+  operand *right;
   sym_link *leftType = operandType (IC_LEFT (ic));
   sym_link *rightType = operandType (IC_RIGHT (ic));
 
   remiCodeFromeBBlock (ebp, ic);
 
+  left = IC_LEFT (ic);
+  right = IC_RIGHT (ic);
+
+  if (IS_SYMOP (left))
+      bitVectUnSetBit (OP_USES (left), ic->key);
+  if (IS_SYMOP (right))
+      bitVectUnSetBit (OP_USES (right), ic->key);
+
   if (getSize (leftType) == 1 && getSize (rightType) == 1)
     {
       int muldivmod;
@@ -703,7 +757,7 @@ convilong (iCode * ic, eBBlock * ebp)
     }
 
   /* depending on the type */
-  for (bwd = 0; bwd < 3; bwd++)
+  for (bwd = 0; bwd < 4; bwd++)
     {
       for (su = 0; su < 2; su++)
         {
@@ -737,7 +791,7 @@ convilong (iCode * ic, eBBlock * ebp)
             }
         }
     }
-  werror (E_INVALID_OP);
+  werrorfl (filename, lineno, E_INVALID_OP, "");
   return;
 found:
   /* if int & long support routines NOT compiled as reentrant */
@@ -757,6 +811,8 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = lineno;
+      if (IS_SYMOP (left))
+          OP_USES (left) = bitVectSetBit (OP_USES (left), newic->key);
 
       /* second one */
       if (IS_REGPARM (FUNC_ARGS(func->type)->next->etype))
@@ -772,6 +828,8 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = lineno;
+      if (IS_SYMOP (right))
+          OP_USES (right) = bitVectSetBit (OP_USES (right), newic->key);
     }
   else
     {
@@ -792,6 +850,8 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = lineno;
+      if (IS_SYMOP (right))
+          OP_USES (right) = bitVectSetBit (OP_USES (right), newic->key);
 
       /* insert push left */
       if (IS_REGPARM (FUNC_ARGS(func->type)->etype))
@@ -809,11 +869,15 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = lineno;
+      if (IS_SYMOP (left))
+          OP_USES (left) = bitVectSetBit (OP_USES (left), newic->key);
     }
 
   /* for the result */
   newic = newiCode (CALL, operandFromSymbol (func), NULL);
   IC_RESULT (newic) = IC_RESULT (ic);
+  bitVectUnSetBit (OP_DEFS (IC_RESULT (ic)), ic->key);
+  OP_DEFS (IC_RESULT (newic)) = bitVectSetBit (OP_DEFS (IC_RESULT (newic)), newic->key);
   newic->filename = filename;
   newic->lineno = lineno;
   newic->parmBytes+=bytesPushed; // to clear the stack after the call
@@ -870,17 +934,30 @@ convbuiltin (iCode *const ic, eBBlock *ebp)
 
   /* Now we can be sure to have found a builtin function. */
 
-  if ((TARGET_IS_Z80 || TARGET_IS_Z180 || TARGET_IS_R2K) && (!strcmp (bif->name, "__builtin_memcpy") || !strcmp (bif->name, "__builtin_memset")))
+  if ((!strcmp (bif->name, "__builtin_memcpy") || !strcmp (bif->name, "__builtin_strncpy") || !strcmp (bif->name, "__builtin_memset")) &&
+    IS_OP_LITERAL (IC_LEFT (lastparam)) && !operandLitValue (IC_LEFT (lastparam)))
+    {
+      /* We have a builtin that does nothing. */
+      /* TODO: Eliminate it, convert any SEND of volatile into DUMMY_READ_VOLATILE. */
+      /* For now just convert back to call to make sure any volatiles are read. */
+
+      strcpy(OP_SYMBOL (IC_LEFT (icc))->rname, !strcmp (bif->name, "__builtin_memcpy") ? "_memcpy" : (!strcmp (bif->name, "__builtin_strncpy") ? "_strncpy" : "_memset"));
+      goto convert;
+    }
+
+  if ((TARGET_IS_Z80 || TARGET_IS_Z180 || TARGET_IS_RABBIT) && (!strcmp (bif->name, "__builtin_memcpy") || !strcmp (bif->name, "__builtin_strncpy") || !strcmp (bif->name, "__builtin_memset")))
     {
       /* Replace iff return value is used or last parameter is not an integer constant. */
       if (bitVectIsZero (OP_USES (IC_RESULT (icc))) && IS_OP_LITERAL (IC_LEFT (lastparam)))
         return;
       
-      strcpy(OP_SYMBOL (IC_LEFT (icc))->rname, !strcmp (bif->name, "__builtin_memcpy") ? "_memcpy" : "_memset");
+      strcpy(OP_SYMBOL (IC_LEFT (icc))->rname, !strcmp (bif->name, "__builtin_memcpy") ? "_memcpy" : (!strcmp (bif->name, "__builtin_strncpy") ? "_strncpy" : "_memset"));
+      goto convert;
     }
-  else
-    return;
+  
+  return;
 
+convert:
   /* Convert parameter passings from SEND to PUSH. */
   stack = 0;
   for (icc = ic; icc->op != CALL; icc = icc->next)
@@ -916,31 +993,32 @@ convbuiltin (iCode *const ic, eBBlock *ebp)
 static void
 convsmallc (iCode *ic, eBBlock *ebp)
 {
-  iCode *icc, *icp, *ico;
+  iCode *icc, *icp, *ico = NULL;
 
   assert (ic->op == CALL || ic->op == PCALL);
 
-  for(icc = ic->prev; icc && icc->op == IPUSH; icc = icc->prev);
+  for (icc = ic->prev; icc && icc->op == IPUSH; icc = icc->prev)
+    ;
   icp = icc;
   ic = icp->next;
 
   /* Reverse parameters. */
   for (icc = ic; icc->op != CALL && icc->op != PCALL; icc = icc->next)
     {
-      if(icc->next->op != CALL && icc->next->op != PCALL)
+      if (icc->next->op != CALL && icc->next->op != PCALL)
         icc->prev = icc->next;
       else
         icc->prev = icp;
     }
-  if(icc != ic)
+  if (icc != ic)
     {
-      if(icp)
+      if (icp)
         icp->next = icc->prev;
       icc->prev = ic;
     }
-  for(; icc != icp; ico = icc, icc = icc->prev)
+  for (; icc != icp; ico = icc, icc = icc->prev)
     {
-      if(icc->op != CALL && icc->op != PCALL)
+      if (icc->op != CALL && icc->op != PCALL)
         icc->next = ico;
     }
 }
@@ -1062,7 +1140,7 @@ convertToFcall (eBBlock ** ebbs, int count)
 /*-----------------------------------------------------------------*/
 /* isPowerOf2 - test if val is power of 2                          */
 /*-----------------------------------------------------------------*/
-static bool
+bool
 isPowerOf2 (unsigned long val)
 {
   while (val && !(val & 1))
@@ -1108,7 +1186,7 @@ miscOpt (eBBlock ** ebbs, int count)
             case LE_OP:
             case '>':
             case GE_OP:
-              /* Only if the the right operand is literal and left operant is unsigned */
+              /* Only if the the right operand is literal and left operand is unsigned */
               if (isOperandLiteral (IC_RIGHT (ic)) && IS_UNSIGNED (operandType (IC_LEFT (ic))))
                 {
                   unsigned litVal = ulFromVal (OP_VALUE (IC_RIGHT (ic)));
@@ -1163,6 +1241,243 @@ miscOpt (eBBlock ** ebbs, int count)
     } /* for */
 }
 
+/*-----------------------------------------------------------------*/
+/* separateAddressSpaces - enforce restrictions on bank switching  */
+/* Operands of a single iCode must be in at most one               */
+/* named address space. Use temporaries and additional assignments */
+/* to enforce the rule.                                            */
+/*-----------------------------------------------------------------*/
+static void
+separateAddressSpaces (eBBlock ** ebbs, int count)
+{
+  
+  int i;
+
+  /* for all blocks do */
+  for (i = 0; i < count; ++i)
+    {
+      iCode *ic;
+      symbol *source;
+
+      /* Skip this block if not reachable; other routines may have */
+      /* also skipped it, so these iCodes may be undercooked. */
+      if (ebbs[i]->noPath)
+        continue;
+
+      /* for all instructions in the block do */
+      for (ic = ebbs[i]->sch; ic; ic = ic->next)
+        {
+          iCode *iic = 0, *newic = 0;
+          operand *left, *right, *result;
+          const symbol *leftaddrspace = 0, *rightaddrspace = 0, *resultaddrspace = 0;
+
+          /* JUMPTABLE and IFX do not have left/right/result operands. */
+          /* However, they only have a single operand so they cannot   */
+          /* have more than one address space to worry about. */
+          if (ic->op == JUMPTABLE || ic->op == IFX)
+            continue;
+
+          left = IC_LEFT (ic);
+          right = IC_RIGHT (ic);
+          result = IC_RESULT (ic);
+          
+          /*printf ("Looking at ic %d, op %d\n", ic->key, (int)(ic->op));*/
+          
+          if (left && IS_SYMOP (left))
+            {
+              if (POINTER_GET (ic))
+                {
+                  assert (!(IS_DECL (OP_SYMBOL (left)->type) && DCL_PTR_ADDRSPACE (OP_SYMBOL (left)->type)));
+                  leftaddrspace = getAddrspace (OP_SYMBOL (left)->type->next);
+                }
+              else
+                leftaddrspace = getAddrspace (OP_SYMBOL (left)->type);
+            }
+          if (right && IS_SYMOP (right))
+            rightaddrspace = getAddrspace (OP_SYMBOL (right)->type);
+          if (result && IS_SYMOP (result))
+            { 
+              if (POINTER_SET (ic))
+                {
+                  assert (!(IS_DECL (OP_SYMBOL (result)->type) && DCL_PTR_ADDRSPACE (OP_SYMBOL (result)->type)));
+                  resultaddrspace = getAddrspace (OP_SYMBOL (result)->type->next);
+                }
+              else
+                resultaddrspace = getAddrspace (OP_SYMBOL (result)->type);
+            }
+            
+#if 0            
+          if (leftaddrspace)
+            printf("ic %d (dcl? %d) leftaddrspace %s\n", ic->key, (int)(IS_DECL  (OP_SYMBOL (left)->type)), leftaddrspace->name);
+          if (rightaddrspace)
+            printf("ic %d (dcl? %d) rightaddrspace %s\n", ic->key, (int)(IS_DECL  (OP_SYMBOL (right)->type)), rightaddrspace->name);
+          if (resultaddrspace)
+            printf("ic %d (dcl? %d) resultaddrspace %s\n", ic->key, (int)(IS_DECL  (OP_SYMBOL (result)->type)), resultaddrspace->name);
+#endif
+            
+          if (ic->op == IPUSH && leftaddrspace)
+            {
+              operand *newop;
+              
+              source = OP_SYMBOL (left); 
+              newic = newiCode ('=', 0, left);
+              IC_RESULT (newic) = newop = newiTempOperand (source->type, 0);          
+              IC_LEFT (ic) = newop;
+              leftaddrspace = 0;
+              for (iic = ic; iic->prev && iic->prev->op == IPUSH; iic = iic->prev);
+            }
+          else if (leftaddrspace && rightaddrspace && leftaddrspace != rightaddrspace)
+            {
+              operand *newop;
+              
+              if (rightaddrspace == resultaddrspace)
+                source = OP_SYMBOL (left);
+              else
+                source = OP_SYMBOL (right);
+              newic = newiCode ('=', 0, rightaddrspace == resultaddrspace ? left : right);
+              IC_RESULT (newic) = newop = newiTempOperand (source->type, 0);
+              if (rightaddrspace == resultaddrspace)
+                {
+                  IC_LEFT (ic) = newop;
+                  leftaddrspace = 0;
+                }
+              else
+                {
+                  IC_RIGHT (ic) = newop;
+                  rightaddrspace = 0;
+                }
+              iic = ic;
+            }
+            
+          if (newic)
+            {
+              newic->filename = ic->filename;
+              newic->lineno = ic->lineno;
+              addiCodeToeBBlock (ebbs[i], newic, iic);
+            } 
+            
+          assert (!leftaddrspace || !resultaddrspace || leftaddrspace == resultaddrspace);
+          assert (!rightaddrspace || !resultaddrspace || rightaddrspace == resultaddrspace);
+        }
+    }
+}
+
+const symbol *
+getAddrspaceiCode (const iCode *ic)
+{
+  operand *left, *right, *result;
+  const symbol *leftaddrspace = 0, *rightaddrspace = 0, *resultaddrspace = 0;
+  const symbol *addrspace;
+
+  /* Not safe to use IC_LEFT, IC_RIGHT, or IC_RESULT macros on */
+  /* IFX or JUMPTABLE iCodes. Handle these as a special case.  */
+  if (ic->op == IFX || ic->op == JUMPTABLE)
+    {
+      operand *cond;
+      if (ic->op == IFX)
+        cond = IC_COND (ic);
+      else
+        cond = IC_JTCOND (ic);
+      if (IS_SYMOP (cond))
+        return getAddrspace (OP_SYMBOL (cond)->type);
+      else
+        return NULL;
+    }
+
+  left = IC_LEFT (ic);
+  right = IC_RIGHT (ic);
+  result = IC_RESULT (ic);
+
+  /* Previous transformations in separateAddressSpaces() should
+     ensure that at most one addressspace occours in each iCode. */
+  if (left && IS_SYMOP (left))
+    { 
+      if (POINTER_GET (ic))
+        {
+          assert (!(IS_DECL (OP_SYMBOL (left)->type) && DCL_PTR_ADDRSPACE (OP_SYMBOL (left)->type)));
+          leftaddrspace = getAddrspace (OP_SYMBOL (left)->type->next);
+        }
+      else
+        leftaddrspace = getAddrspace (OP_SYMBOL (left)->type);
+    }
+  if (right && IS_SYMOP (right))
+    rightaddrspace = getAddrspace (OP_SYMBOL (right)->type);
+  if (result && IS_SYMOP (result))
+    { 
+      if (POINTER_SET (ic))
+        {
+          assert (!(IS_DECL (OP_SYMBOL (result)->type) && DCL_PTR_ADDRSPACE (OP_SYMBOL (result)->type)));
+          resultaddrspace = getAddrspace (OP_SYMBOL (result)->type->next);
+        }
+      else
+        resultaddrspace = getAddrspace (OP_SYMBOL (result)->type);
+    }
+           
+  addrspace = leftaddrspace;
+  if (rightaddrspace)
+    {
+      wassertl (!addrspace || addrspace == rightaddrspace, "Multiple named address spaces in icode.");
+      addrspace = rightaddrspace;
+    }
+  if (resultaddrspace)
+    {
+      wassertl (!addrspace || addrspace == resultaddrspace, "Multiple named address spaces in icode.");
+      addrspace = resultaddrspace;
+    }
+
+  return (addrspace);
+}
+
+/*-----------------------------------------------------------------*/
+/* switchAddressSpaceAt - insert a bank selection instruction      */
+/*-----------------------------------------------------------------*/
+void
+switchAddressSpaceAt (iCode *ic, const symbol *const addrspace)
+{
+  iCode *newic;
+  const symbol *const laddrspace = getAddrspaceiCode (ic);
+  wassertl(!laddrspace || laddrspace == addrspace, "Switching to invalid address space.");
+
+  newic = newiCode (CALL, operandFromSymbol (addrspace->addressmod[0]), 0);
+
+  IC_RESULT (newic) = newiTempOperand (newVoidLink (), 1);
+  newic->filename = ic->filename;
+  newic->lineno = ic->lineno;
+
+  newic->next = ic;
+  newic->prev = ic->prev;
+  if (ic->prev)
+    ic->prev->next = newic;
+  ic->prev = newic;
+}
+
+/*-----------------------------------------------------------------*/
+/* switchAddressSpaces - insert instructions for bank switching    */
+/* This is just a fallback, in case the optimal approach fails -   */
+/* improbable, but possible depending on sdcc options and code.    */
+/*-----------------------------------------------------------------*/
+static void
+switchAddressSpaces (iCode *ic)
+{
+  const symbol *oldaddrspace = 0;
+
+  for (; ic; ic = ic->next)
+    {
+      const symbol *const addrspace = getAddrspaceiCode (ic);
+ 
+      if (addrspace && addrspace != oldaddrspace)
+        { 
+          switchAddressSpaceAt (ic, addrspace);
+          
+          oldaddrspace = addrspace;
+        }
+        
+      /* Address space might not be preserved over these. */
+      if (ic->op == LABEL || ic->op == CALL || ic->op == PCALL)
+        oldaddrspace = 0;
+    }
+}
+
 /*-----------------------------------------------------------------*/
 /* isLocalWithoutDef - return 1 if sym might be used without a     */
 /*                     defining iCode                              */
@@ -1207,6 +1522,9 @@ replaceRegEqv (ebbIndex * ebbi)
   for (i = 0; i < count; i++)
     {
       iCode *ic;
+      
+      if (ebbs[i]->noPath)
+        continue;
 
       for (ic = ebbs[i]->sch; ic; ic = ic->next)
         {
@@ -1428,6 +1746,10 @@ killDeadCode (ebbIndex * ebbi)
               if (IC_RESULT (ic) && POINTER_SET (ic))
                 continue;
 
+              /* if the results address has been taken then skip */
+              if (IS_SYMOP (IC_RESULT (ic)) && OP_SYMBOL (IC_RESULT (ic))->addrtaken)
+                continue;
+
               if (POINTER_GET (ic) && IS_VOLATILE (operandType (IC_LEFT (ic))->next)
                   && !SPIL_LOC (IC_RESULT (ic)))
                 continue;
@@ -1639,6 +1961,7 @@ optimizeCastCast (eBBlock ** ebbs, int count)
   sym_link * type2;
   sym_link * type3;
   symbol * sym;
+  int size1, size2, size3;
 
   for (i = 0; i < count; i++)
     {
@@ -1653,7 +1976,12 @@ optimizeCastCast (eBBlock ** ebbs, int count)
               /* integer type that has no loss of bits */
               if (!IS_INTEGRAL (type1) || !IS_INTEGRAL (type2))
                 continue;
-              if (getSize (type2) < getSize (type1))
+              size1 = bitsForType (type1);
+              size2 = bitsForType (type2);
+              if (size2 < size1)
+                continue;
+              /* If they are the same size, they must have the same signedness */
+              if (size2 == size1 && SPEC_USIGN (type2) != SPEC_USIGN (type1))
                 continue;
 
               /* There must be only one use of this first result */
@@ -1671,8 +1999,12 @@ optimizeCastCast (eBBlock ** ebbs, int count)
               type3 = operandType (IC_RESULT (uic));
               if (!IS_INTEGRAL (type3))
                  continue;
-              if (getSize (type3) < getSize (type2))
+              size3 = bitsForType (type3);
+              if (size3 < size2)
                  continue;
+              /* If they are the same size, they must have the same signedness */
+              if (size3 == size2 && SPEC_USIGN (type3) != SPEC_USIGN (type2))
+                continue;
 
               /* The signedness between the first and last types */
               /* must match */
@@ -1695,6 +2027,55 @@ optimizeCastCast (eBBlock ** ebbs, int count)
     }
 }
 
+/* Fold pointer addition into offset of GET_VALUE_AT_ADDRESS.                  */
+/* The hc08-related ports do a similar thing in hc08/ralloc.c, packPointerOp() */
+static void
+offsetFold (eBBlock **ebbs, int count)
+{
+  int i;
+  iCode *ic;
+  iCode *uic;
+
+  if (!TARGET_IS_Z80 && !TARGET_IS_Z180 && !TARGET_IS_RABBIT)
+    return;
+
+  for (i = 0; i < count; i++)
+    {
+      for (ic = ebbs[i]->sch; ic; ic = ic->next)
+        {
+          if ((ic->op == '+' || ic->op == '-') && IC_RESULT (ic) && IS_ITEMP (IC_RESULT (ic)))
+            {
+              if (!IS_OP_LITERAL (IC_RIGHT (ic)))
+                continue;
+
+              /* There must be only one use of the result */
+              if (bitVectnBitsOn (OP_USES (IC_RESULT (ic))) != 1)
+                continue;
+
+              /* This use must be a GET_VALUE_AT_DDRESS */
+              uic = hTabItemWithKey (iCodehTab,
+                        bitVectFirstBit (OP_USES (IC_RESULT (ic))));
+              if (!POINTER_GET (uic))
+                continue;
+
+              /* Historically GET_VALUE_AT_ADDRESS didn't have a right operand */
+              wassertl (IC_RIGHT (uic), "GET_VALUE_AT_ADDRESS without right operand");
+              wassertl (IS_OP_LITERAL (IC_RIGHT (uic)), "GET_VALUE_AT_ADDRESS with non-literal right operand");
+
+              if (ic->op == '+')
+                IC_RIGHT (uic) = operandFromLit (operandLitValue (IC_RIGHT (uic)) + operandLitValue (IC_RIGHT (ic)));
+              else
+                IC_RIGHT (uic) = operandFromLit (operandLitValue (IC_RIGHT (uic)) - operandLitValue (IC_RIGHT (ic)));
+
+              ic->op = '=';
+              IC_RIGHT (ic) = IC_LEFT (ic);
+              IC_LEFT (ic) = 0;
+              SET_ISADDR (IC_RESULT (ic), 0);
+            }
+        }
+    }
+}
+
 /*-----------------------------------------------------------------*/
 /* eBBlockFromiCode - creates extended basic blocks from iCode     */
 /*                    will return an array of eBBlock pointers     */
@@ -1721,7 +2102,6 @@ eBBlockFromiCode (iCode * ic)
 
   /* break it down into basic blocks */
   ebbi = iCodeBreakDown (ic);
-
   /* hash the iCode keys so that we can quickly index */
   /* them in the rest of the optimization steps */
   setToNull ((void *) &iCodehTab);
@@ -1730,7 +2110,7 @@ eBBlockFromiCode (iCode * ic)
 
   /* compute the control flow */
   computeControlFlow (ebbi);
-
+ 
   /* dumpraw if asked for */
   if (options.dump_raw)
     dumpEbbsToFileExt (DUMP_RAW0, ebbi);
@@ -1781,6 +2161,8 @@ eBBlockFromiCode (iCode * ic)
       // compute the dataflow only
       assert(cseAllBlocks (ebbi, TRUE)==0);
     }
+
+
   /* kill dead code */
   kchange = killDeadCode (ebbi);
 
@@ -1816,9 +2198,42 @@ eBBlockFromiCode (iCode * ic)
         dumpEbbsToFileExt (DUMP_LOOPD, ebbi);
     }
 
+  offsetFold (ebbi->bbOrder, ebbi->count);
+
+  /* lospre */
+  adjustIChain (ebbi->bbOrder, ebbi->count);
+  ic = iCodeLabelOptimize (iCodeFromeBBlock (ebbi->bbOrder, ebbi->count));
+  if (optimize.lospre && (TARGET_Z80_LIKE || TARGET_HC08_LIKE)) /* Todo: enable for other ports. */
+    lospre (ic, ebbi);
+  /* Break down again and redo some steps to not confuse live range analysis. */
+  ebbi = iCodeBreakDown (ic);
+  computeControlFlow (ebbi);
+  loops = createLoopRegions (ebbi);
+  computeDataFlow (ebbi);
+
+  killDeadCode (ebbi);
+
   /* sort it back by block number */
   //qsort (ebbs, saveCount, sizeof (eBBlock *), bbNumCompare);
 
+  /* enforce restrictions on acesses to named address spaces */
+  separateAddressSpaces (ebbi->bbOrder, ebbi->count);
+
+  /* insert bank switching instructions. Do it here, before the
+     other support routines, since we can assume that there is no
+     bank switching happening in those other support routines
+     (but assume that it can happen in other functions) */
+  adjustIChain (ebbi->bbOrder, ebbi->count);
+  ic = iCodeLabelOptimize (iCodeFromeBBlock (ebbi->bbOrder, ebbi->count));
+  if (switchAddressSpacesOptimally (ic, ebbi))
+    switchAddressSpaces (ic); /* Fallback. Very unlikely to be triggered, unless --max-allocs-per-node is set to very small values or very weird control-flow graphs */
+
+  /* Break down again and redo some steps to not confuse live range analysis. */
+  ebbi = iCodeBreakDown (ic);
+  computeControlFlow (ebbi);
+  loops = createLoopRegions (ebbi);
+  computeDataFlow (ebbi);
+
   if (!options.lessPedantic)
     {
       // this is a good place to check missing return values
@@ -1874,3 +2289,4 @@ eBBlockFromiCode (iCode * ic)
 
   return NULL;
 }
+
