@@ -2,25 +2,21 @@
   SDCCpeeph.c - The peep hole optimizer: for interpreting the
                 peep hole rules
 
-             Written By -  Sandeep Dutta . sandeep.dutta@usa.net (1999)
+  Copyright (C) 1999, Sandeep Dutta . sandeep.dutta@usa.net
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; either version 2, or (at your option) any
-   later version.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2, or (at your option) any
+  later version.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-
-   In other words, you are welcome to use, share and improve this program.
-   You are forbidden to forbid anyone else to use, share and improve
-   what you give them.   Help stamp out software-hoarding!
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 -------------------------------------------------------------------------*/
 
 #include "common.h"
@@ -312,7 +308,7 @@ FBYNAME (labelIsReturnOnly)
     ;
 
   retInst = "ret";
-  if (TARGET_IS_HC08)
+  if (TARGET_HC08_LIKE)
     retInst = "rts";
   if (strcmp(p, retInst) == 0)
     return TRUE;
@@ -367,8 +363,11 @@ FBYNAME (labelIsUncondJump)
 
   if (TARGET_MCS51_LIKE)
     jpInst = "ljmp";
-  if (TARGET_IS_HC08)
-    jpInst = "jmp";
+  if (TARGET_HC08_LIKE)
+    {
+      jpInst = "jmp";
+      jpInst2 = "bra";
+    }
   if (TARGET_Z80_LIKE)
     {
       jpInst = "jp";
@@ -515,9 +514,8 @@ FBYNAME (labelRefCount)
             }
           else
             {
-              fprintf (stderr, "*** internal error: no label has entry for"
-                       " %s in labelRefCount peephole.\n",
-                       label);
+              // Not a local label. We do not know how often it might be referenced.
+              rc = FALSE;
             }
         }
       else
@@ -592,9 +590,8 @@ FBYNAME (labelRefCountChange)
             }
             else
             {
-              fprintf (stderr, "*** internal error: no label has entry for"
-                       " %s in %s peephole.\n",
-                       label, __FUNCTION__);
+              // Not a local label. We do not know how often it might be referenced.
+              return TRUE;
             }
         }
       else
@@ -907,6 +904,19 @@ operandBaseName (const char *op)
       if (op[0] == '@')
         return operandBaseName(op+1);
     }
+  if (TARGET_Z80_LIKE)
+    {
+      if (!strcmp (op, "d") || !strcmp (op, "e") || !strcmp (op, "(de)"))
+        return "de";
+      if (!strcmp (op, "b") || !strcmp (op, "c") || !strcmp (op, "(bc)"))
+        return "bc";
+      if (!strcmp (op, "h") || !strcmp (op, "l") || !strcmp (op, "(hl)") || !strcmp (op, "(hl+)")  || !strcmp (op, "(hl-)"))
+        return "hl";
+      if (!strcmp (op, "iyh") || !strcmp (op, "iyl") || strstr (op, "iy"))
+        return "iy";
+      if (!strcmp (op, "ixh") || !strcmp (op, "ixl") || strstr (op, "ix"))
+        return "ix";
+    }
 
   return op;
 }
@@ -917,6 +927,8 @@ operandBaseName (const char *op)
 FBYNAME (notUsed)
 {
   const char *what;
+  bool ret;
+
   set *operands = setFromConditionArgs (cmdLine, vars);
 
   if (!operands || elementsInSet(operands) != 1)
@@ -929,11 +941,17 @@ FBYNAME (notUsed)
 
   what = setFirstItem (operands);
 
-  if (port->peep.notUsed)
-    return port->peep.notUsed (what, endPl, head);
+  if (!port->peep.notUsed)
+    {
+      fprintf (stderr, "Function notUsed not initialized in port structure\n");
+      return FALSE;
+    }
 
-  fprintf (stderr, "Function notUsed not initialized in port structure\n");
-  return FALSE;
+  ret = port->peep.notUsed (what, endPl, head);
+
+  deleteSet(&operands);
+
+  return (ret);
 }
 
 /*-----------------------------------------------------------------*/
@@ -1103,7 +1121,7 @@ FBYNAME (operandsLiteral)
 
   for (op = setFirstItem (operands); op; op = setNextItem (operands))
     {
-      if (!isdigit(*op))
+      if (!isdigit( (unsigned char)(*op) ))
         {
           deleteSet (&operands);
           return FALSE;
@@ -1284,45 +1302,6 @@ callFuncByName (char *fname,
   return rc;
 }
 
-/*-----------------------------------------------------------------*/
-/* printLine - prints a line chain into a given file               */
-/*-----------------------------------------------------------------*/
-void
-printLine (lineNode * head, struct dbuf_s * oBuf)
-{
-  iCode *last_ic = NULL;
-  bool debug_iCode_tracking = (getenv("DEBUG_ICODE_TRACKING")!=NULL);
-
-  while (head)
-    {
-      if (head->ic!=last_ic)
-        {
-          last_ic = head->ic;
-          if (debug_iCode_tracking)
-            {
-              if (head->ic)
-                dbuf_printf (oBuf, "; block = %d, seq = %d\n",
-                         head->ic->block, head->ic->seq);
-              else
-                dbuf_append_str (oBuf, "; iCode lost\n");
-            }
-        }
-
-      /* don't indent comments & labels */
-      if (head->line &&
-          (head->isComment || head->isLabel)) {
-        dbuf_printf (oBuf, "%s\n", head->line);
-      } else {
-        if (head->isInline && *head->line=='#') {
-          // comment out preprocessor directives in inline asm
-          dbuf_append_char (oBuf, ';');
-        }
-        dbuf_printf (oBuf, "\t%s\n", head->line);
-      }
-      head = head->next;
-    }
-}
-
 /*-----------------------------------------------------------------*/
 /* newPeepRule - creates a new peeprule and attach it to the root  */
 /*-----------------------------------------------------------------*/
@@ -1359,38 +1338,6 @@ newPeepRule (lineNode * match,
   return pr;
 }
 
-/*-----------------------------------------------------------------*/
-/* newLineNode - creates a new peep line                           */
-/*-----------------------------------------------------------------*/
-lineNode *
-newLineNode (const char *line)
-{
-  lineNode *pl;
-
-  pl = Safe_alloc ( sizeof (lineNode));
-  pl->line = Safe_strdup (line);
-  pl->ic = NULL;
-  return pl;
-}
-
-/*-----------------------------------------------------------------*/
-/* connectLine - connects two lines                                */
-/*-----------------------------------------------------------------*/
-lineNode *
-connectLine (lineNode * pl1, lineNode * pl2)
-{
-  if (!pl1 || !pl2)
-    {
-      fprintf (stderr, "trying to connect null line\n");
-      return NULL;
-    }
-
-  pl2->prev = pl1;
-  pl1->next = pl2;
-
-  return pl2;
-}
-
 #define SKIP_SPACE(x,y) { while (*x && (ISCHARSPACE(*x) || *x == '\n')) x++; \
                          if (!*x) { fprintf(stderr,y); return ; } }
 
@@ -2105,7 +2052,7 @@ isLabelDefinition (const char *line, const char **start, int *len,
 {
   const char *cp = line;
 
-  /* This line is a label if if consists of:
+  /* This line is a label if it consists of:
    * [optional whitespace] followed by identifier chars
    * (alnum | $ | _ ) followed by a colon.
    */
