@@ -26,18 +26,6 @@
 #include "SDCCsymt.h"
 
 
-/* double to unsigned long conversion */
-/*
- * See ISO/IEC 9899, chapter 6.3.1.4 Real floating and integer:
- * If the value of the integral part cannot be represented by the integer type, the behavior is undefined.
- * This shows up on Mac OS X i386 platform which useus SSE unit instead of the x87 FPU for floating-point operations
- */
-/*
- * on Mac OS X ppc (long) 2147483648.0 equals to 2147483647, so we explicitely convert it to 0x80000000
- * on other known platforms (long) 2147483648.0 equals to -2147483648
- */
-#define double2ul(val)  (((val) < 0) ? (((val) < -2147483647.0) ? 0x80000000UL : (unsigned long) -((long) -(val))) : (unsigned long) (val))
-
 /* value wrapper */
 typedef struct value
 {
@@ -96,8 +84,8 @@ typedef struct designation
     char *filename;
     union
       {
-	struct symbol *tag;             /* tag part of structure          */
-	int elemno;                     /* array element (constant expr)  */
+        struct symbol *tag;             /* tag part of structure          */
+        int elemno;                     /* array element (constant expr)  */
       }
     designator;
 
@@ -121,6 +109,7 @@ CCR_RESULT;
 value *newValue (void);
 value *constVal (const char *);
 value *constCharVal (unsigned char);
+value *constBoolVal (bool v);
 value *reverseVal (value *);
 value *reverseValWithType (value *);
 value *copyValue (value *);
@@ -175,4 +164,10 @@ value *valForCastAggr (struct ast *, sym_link *, struct ast *, int);
 value *valForCastArr (struct ast *, sym_link *);
 bool convertIListToConstList (initList * src, literalList ** lList, int size);
 literalList *copyLiteralList (literalList * src);
+unsigned long double2ul (double val);
+unsigned char byteOfVal (value *, int);
+int isEqualVal (value *, int);
+TYPE_TARGET_ULONGLONG ullFromLit (sym_link * lit);
+value * valRecastLitVal (sym_link * dtype, value * val);
+
 #endif
