@@ -1,6 +1,25 @@
-/** @file main.c
-    mcs51 specific general functions.
+/*-------------------------------------------------------------------------
+  main.c - mcs51 specific general functions
 
+  Copyright (C) 1998, Sandeep Dutta . sandeep.dutta@usa.net
+  Copyright (C) 1999, Jean-Louis VERN.jlvern@writeme.com
+  Copyright (C) 2000, Michael Hope
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2, or (at your option) any
+  later version.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+-------------------------------------------------------------------*/
+/*
     Note that mlh prepended _mcs51_ on the static functions.  Makes
     it easier to set a breakpoint using the debugger.
 */
@@ -26,6 +45,7 @@ static OPTION _mcs51_options[] =
     { 0, "--pack-iram",      NULL, "Tells the linker to pack variables in internal ram (default)"},
     { 0, "--no-pack-iram",   &options.no_pack_iram, "Deprecated: Tells the linker not to pack variables in internal ram"},
     { 0, "--acall-ajmp",     &options.acall_ajmp, "Use acall/ajmp instead of lcall/ljmp" },
+    { 0, "--no-ret-without-call", &options.no_ret_without_call, "Do not use ret independent of acall/lcall" },
     { 0, NULL }
   };
 
@@ -102,7 +122,7 @@ _mcs51_regparm (sym_link * l, bool reentrant)
   else
     {
       int size = getSize(l);
-      int remain ;
+      int remain;
 
       /* first one goes the usual way to DPTR */
       if (regParmFlg == 0)
@@ -683,7 +703,7 @@ static int
 getInstructionSize (lineNode *line)
 {
   if (!line->aln)
-    line->aln = asmLineNodeFromLineNode (line);
+    line->aln = (asmLineNodeBase *) asmLineNodeFromLineNode (line);
 
   return line->aln->size;
 }
@@ -692,7 +712,7 @@ static bitVect *
 getRegsRead (lineNode *line)
 {
   if (!line->aln)
-    line->aln = asmLineNodeFromLineNode (line);
+    line->aln = (asmLineNodeBase *) asmLineNodeFromLineNode (line);
 
   return line->aln->regsRead;
 }
@@ -701,7 +721,7 @@ static bitVect *
 getRegsWritten (lineNode *line)
 {
   if (!line->aln)
-    line->aln = asmLineNodeFromLineNode (line);
+    line->aln = (asmLineNodeBase *) asmLineNodeFromLineNode (line);
 
   return line->aln->regsWritten;
 }
@@ -752,13 +772,13 @@ get_model (void)
 */
 static const char *_linkCmd[] =
 {
-  "sdld", "-nf", "\"$1\"", NULL
+  "sdld", "-nf", "$1", NULL
 };
 
 /* $3 is replaced by assembler.debug_opts resp. port->assembler.plain_opts */
 static const char *_asmCmd[] =
 {
-  "sdas8051", "$l", "$3", "\"$2\"", "\"$1.asm\"", NULL
+  "sdas8051", "$l", "$3", "$2", "$1.asm", NULL
 };
 
 static const char * const _libs[] = { "mcs51", STD_LIB, STD_INT_LIB, STD_LONG_LIB, STD_FP_LIB, NULL, };
@@ -780,8 +800,8 @@ PORT mcs51_port =
   {                             /* Assembler */
     _asmCmd,
     NULL,
-    "-plosgffwzc",              /* Options with debug */
-    "-plosgffwz",               /* Options without debug */
+    "-plosgffwy",               /* Options with debug */
+    "-plosgffw",                /* Options without debug */
     0,
     ".asm",
     NULL                        /* no do_assemble function */
@@ -830,9 +850,12 @@ PORT mcs51_port =
     "CABS    (ABS,CODE)",       // cabs_name - const absolute data (code or not)
     "XABS    (ABS,XDATA)",      // xabs_name - absolute xdata/pdata
     "IABS    (ABS,DATA)",       // iabs_name - absolute idata/data
+    NULL,                       // name of segment for initialized variables
+    NULL,                       // name of segment for copies of initialized variables in code space
     NULL,
     NULL,
-    1
+    1,
+    1                           // No fancy alignments supported.
   },
   { _mcs51_genExtraAreas, NULL },
   {
@@ -893,6 +916,7 @@ PORT mcs51_port =
   NULL,                         /* no builtin functions */
   GPOINTER,                     /* treat unqualified pointers as "generic" pointers */
   1,                            /* reset labelKey to 1 */
-  1,                            /* globals & local static allowed */
+  1,                            /* globals & local statics allowed */
+  0,                            /* Number of registers handled in the tree-decomposition-based register allocator in SDCCralloc.hpp */
   PORT_MAGIC
 };
