@@ -22,6 +22,7 @@
   what you give them.   Help stamp out software-hoarding!
 -------------------------------------------------------------------------*/
 
+#include <ctype.h>
 #include "common.h"
 #include "ralloc.h"
 
@@ -219,7 +220,7 @@ isFunc (const lineNode *pl)
 /* termScanAtFunc - returns S4O_TERM if it's a 'normal' function   */
 /* call and it's a 'caller save'. returns S4O_CONTINUE if it's     */
 /* 'callee save' or 'naked'. returns S4O_ABORT if it's 'banked'    */
-/* uses the register for the destination.                          */
+/* and uses the register for the destination.                      */
 /*-----------------------------------------------------------------*/
 static S4O_RET
 termScanAtFunc (const lineNode *pl, int rIdx)
@@ -444,6 +445,20 @@ scan4op (lineNode **pl, const char *pReg, const char *untilOp,
           case 'l':
             if (strncmp ("lcall", (*pl)->line, 5) == 0)
               {
+                const char *p = (*pl)->line+5;
+                while (*p == ' ' || *p == '\t')
+                  p++;
+                while (isdigit (*p))
+                  p++;
+                if (isdigit(p[-1]) && *p == '$') /* at least one digit */
+                  {
+                    /* this is a temp label for a pcall */
+                    *pl = findLabel (*pl);
+                    if (!*pl)
+                      return S4O_ABORT;
+                    break;
+                  }
+
                 ret = termScanAtFunc (*pl, rIdx);
                 /* If it's a 'normal' 'caller save' function call, all
                    registers have been saved until the 'lcall'. The
@@ -632,7 +647,7 @@ removeDeadPopPush (const char *pReg, lineNode *currPl, lineNode *head)
 
       ; An "acall", "lcall" (not callee save), "ret" (not PCALL with
       ; callee save), "reti" or write access of r0 terminate
-      ; the search, and the "mov r0,a" can safely be removed.
+      ; the search, and the "pop/push ar0" can safely be removed.
   */
 
   /* area 1 */
