@@ -12,6 +12,7 @@ typedef enum
     SUB_Z80,
     SUB_Z180,
     SUB_R2K,
+    SUB_R3KA,
     SUB_GBZ80
   }
 Z80_SUB_PORT;
@@ -23,8 +24,7 @@ typedef struct
     int port_mode;
     int port_back;
     int reserveIY;
-    int dump_graphs;
-    int oldralloc;
+    int noOmitFramePtr;
   }
 Z80_OPTS;
 
@@ -32,16 +32,15 @@ extern Z80_OPTS z80_opts;
 
 #define IS_Z80 (z80_opts.sub == SUB_Z80)
 #define IS_Z180 (z80_opts.sub == SUB_Z180)
-#define IS_GB  (z80_opts.sub == SUB_GBZ80)
-#define IS_R2K  (z80_opts.sub == SUB_R2K)
+#define IS_GB (z80_opts.sub == SUB_GBZ80)
+#define IS_R2K (z80_opts.sub == SUB_R2K)
+#define IS_R3KA (z80_opts.sub == SUB_R3KA)
+#define IS_RAB (IS_R2K || IS_R3KA)
 
 #define IY_RESERVED (z80_opts.reserveIY)
 
-#define OPTRALLOC_REMAT (IS_Z80 || IS_Z180 || IS_R2K) /* Enable rematerialization in the new allocator for the ports with exact cost function */
-#define OPTRALLOC_EXACT_COST (!IS_GB) // Todo: Implement exact cost for gbz80.
 #define OPTRALLOC_HL (!IS_GB)
-#define OPTRALLOC_A 1
-#define OPTRALLOC_IY !IY_RESERVED
+#define OPTRALLOC_IY !(IY_RESERVED || IS_GB)
 
 enum
   {
