@@ -22,7 +22,7 @@ g (int (*h) (int))
 void
 f1 ()
 {
-#if defined(SDCC_ds390)
+#if defined(__SDCC_ds390)
   p = (void __code *) mul2;
 #else
   p = (void *) mul2;
