@@ -4,7 +4,7 @@
 
 #include <testfwk.h>
 
-#if defined (SDCC_ds390) || defined (SDCC_mcs51)
+#if defined (__SDCC_ds390) || defined (__SDCC_mcs51)
   volatile __xdata __at 0x7654 char x;
 #endif
 
@@ -13,9 +13,10 @@ test_volatile (void)
 {
   ASSERT (1);
 
-#if defined (SDCC_ds390) || defined (SDCC_mcs51)
+#if defined (__SDCC_ds390) || defined (__SDCC_mcs51)
   x;         //this should end the simulation
 
   while (1); //let the "watchdog" bite
 #endif
 }
+
