@@ -9,12 +9,13 @@ bug_2805536(void)
 {
 #ifndef PORT_HOST
 __asm
-; != <-- this should not causes a compiler internal error
+; != <-- this should not cause a compiler internal error
 __endasm;
 #endif
+  ASSERT(1);
 }
 
-static void 
+static void
 test_macro(void)
 {
   bug_2805536();
